@@ -90,8 +90,6 @@ from os.path import join, isdir, exists
 from logging import getLogger
 from warnings import warn
 
-from six import string_types, add_metaclass
-
 from logilab.common.modutils import modpath_from_file
 from logilab.common.logging_ext import set_log_methods
 from logilab.common.decorators import classproperty
@@ -317,7 +315,7 @@ class Registry(dict):
         # XXXFIXME this is a duplication of unregister()
         # remove register_and_replace in favor of unregister + register
         # or simplify by calling unregister then register here
-        if not isinstance(replaced, string_types):
+        if not isinstance(replaced, str):
             replaced = self.objid(replaced)
         # prevent from misspelling
         assert obj is not replaced, 'replacing an object by itself: %s' % obj
@@ -408,23 +406,56 @@ class Registry(dict):
         return None if not object apply (don't raise `NoSelectableObject` since
         it's costly when searching objects using `possible_objects`
         (e.g. searching for hooks).
+
+        An optional argument `debug_callback` can be passed, it will be called
+        once the selection is done and will return a dict of the following form::
+
+          {"all_objects": [], "end_score": 0, "winners": [],
+          "self": self, "args": args, "kwargs": kwargs, }
         """
+
+        debug_callback = kwargs.pop('debug_callback', None)
+
+        debug_registry_select_best = {
+            "all_objects": [],
+            "end_score": 0,
+            "winners": [],
+            "self": self,
+            "args": args,
+            "kwargs": kwargs,
+        }
+
         score, winners = 0, None
         for obj in objects:
             objectscore = obj.__select__(obj, *args, **kwargs)
+
+            if debug_callback:
+                debug_registry_select_best["all_objects"].append({"object": obj, "score": objectscore})
+
             if objectscore > score:
                 score, winners = objectscore, [obj]
+
             elif objectscore > 0 and objectscore == score:
                 winners.append(obj)
+
+        if debug_callback:
+            debug_registry_select_best["winners"] = winners.copy() if winners else []
+            debug_registry_select_best["end_score"] = score
+            debug_callback(debug_registry_select_best)
+
         if winners is None:
             return None
+
         if len(winners) > 1:
             # log in production environement / test, error while debugging
             msg = 'select ambiguity: %s\n(args: %s, kwargs: %s)'
+
             if self.debugmode:
                 # raise bare exception in debug mode
                 raise SelectAmbiguity(msg % (winners, args, kwargs.keys()))
+
             self.error(msg, winners, args, kwargs.keys())
+
         # return the result of calling the object
         return self.selected(winners[0], args, kwargs)
 
@@ -623,7 +654,7 @@ class RegistryStore(dict):
         :meth:`~logilab.common.registry.RegistryStore.register_and_replace` for
         instance).
         """
-        assert isinstance(modname, string_types), \
+        assert isinstance(modname, str), \
             'modname expected to be a module name (ie string), got %r' % modname
         for obj in objects:
             if self.is_registrable(obj) and obj.__module__ == modname and not obj in butclasses:
@@ -985,8 +1016,7 @@ class PredicateMetaClass(type):
         return inst
 
 
-@add_metaclass(PredicateMetaClass)
-class Predicate(object):
+class Predicate(object, metaclass=PredicateMetaClass):
     """base class for selector classes providing implementation
     for operators ``&``, ``|`` and  ``~``
 
